import * as Messages from "/services/messages/messages.js";

let sectionModel;

export function getSectionModel() {
  if (sectionModel == null) {
    sectionModel = new SectionModel();
  }
  return sectionModel;
}

class SectionModel {
  constructor() {}

  async getSection(id) {
    if (this.sections == null) await this.refreshSections();
    let sectionToGet;
    this.sections.forEach((section) => {
      if (section.ID == id) sectionToGet = section;
    });
    return sectionToGet;
  }

  async getSections() {
    if (this.sections == null) {
      try {
        const response = await fetch("/api/Section/", {
          method: "GET",
          headers: new Headers({
            "XSRF-Token": this.current_user.xsrftoken,
          }),
        });
        if (response.status !== 200) {
          throw new Error(
            `Sections could not be fetched (status ${response.status})`
          );
        }
        this.sections = await response.json();
      } catch (e) {
        Messages.Show("is-warning", e.message);
        console.error(e);
      }
    }
    return this.sections;
  }

  async saveSection(method, ID, AreaID, Name, MapID) {
    try {
      const response = await fetch("/api/Section/" + ID, {
        method: method,
        headers: new Headers({
          "XSRF-Token": this.current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: ID,
          AreaID: AreaID,
          Name: Name,
          MapID: MapID,
        }),
      });
      if (response.status !== 200) {
        throw new Error(
          `Section could not be updated or created (status ${response.status})`
        );
      }
      this.refreshSections();
      return await response.json();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
      return;
    }
  }

  async deleteSections(ID) {
    try {
      const response = await fetch("/api/Section/" + ID, {
        method: "delete",
        headers: new Headers({
          "XSRF-Token": this.current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(
          `Section could not be deleted (status ${response.status})`
        );
      }
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }

  async refreshSections() {
    this.sections = null;
    await this.getSections();
  }
}
